// This file is part of FreeFinesse, a double-dummy analyzer (c) Ed Colley, 2006
// It is made available under the GPL; see the file COPYING for details

//
//  The interface to the double-dummy solving code
//

// This class is used for a single problem only. For analysing the next trick or alternative plays, a new
// instance of the analyzer must be created
class analyzer {
public:
	// Interface for the Controller
	analyzer(DealController_Analyzer*, const deal_t&, cache_t*);
	bool playCard(card_t);
	void analyze(bool analyzeAllOptions);
	
	// This information is maintained by the analyzer for the UI on the current state of the deal;
	// it is not use or modified by the searching code
	player_t plnext_ui;			// next player
	trickstate_t trickstate_ui;	// state of the current trick
	rankequiv_t rankequiv_ui;	// rank equivalences
	
private:
	// Core searching methods
	const deal_t deal;
	gamestate_t state;
	cache_t* const cache;
	bool search(uint tricktarget, player_t, card_t* winmove);
	bool search_pl0(uint tricktarget, player_t, uint64& rwmask, card_t* winmove, const rankequiv_t&);
	bool search_pl1(uint tricktarget, player_t, uint64& rwmask, card_t* winmove, const rankequiv_t&, const trickstate_t&);
	bool search_pl2(uint tricktarget, player_t, uint64& rwmask, card_t* winmove, const rankequiv_t&, const trickstate_t&);
	bool search_pl3(uint tricktarget, player_t, uint64& rwmask, card_t* winmove, const rankequiv_t&, const trickstate_t&);
	bool search_t13(player_t, uint64& rwmask);
	int sureTricks(player_t pl, card_t* winmove = 0);
	card_t unplayCard();
	
	// Controller interface
	DealController_Analyzer* const dc;
};
