// This file is part of FreeFinesse, a double-dummy analyzer (c) Ed Colley, 2006
// It is made available under the GPL; see the file COPYING for details

//
//  Cache for storing results to avoid repeat computation
//

#include "stdafx.h"

// Check the cache for a given target. Returns -1 (miss), +1 (hit) or 0 (don't know)
int cache_t::check(const gamestate_t& state, player_t pl, uint trickTarget, uint64& rwmask)
{
	cache_resl& resl = data[state.nCardsPlayed/4][pl][state.uSuitLengths];
	for (cache_resl::reverse_iterator it = resl.rbegin(); it != resl.rend(); it++) {
		if ((it->rwMask & it->cardsLeft) == (it->rwMask & state.mCardsLeft)) {
			if (trickTarget <= it->lowerbound) { rwmask |= it->rwMask; return +1; }
			if (trickTarget >= it->upperbound) { rwmask |= it->rwMask; return -1; }
		}
	}
	return 0;
}

// Update when successfully hit trick target
void cache_t::update_hit(const gamestate_t& state, player_t pl, uint64 rwmask, uint trickTarget)
{
	cache_resl& resl = data[state.nCardsPlayed/4][pl][state.uSuitLengths];
	result res;
	res.cardsLeft = state.mCardsLeft;
	res.lowerbound = trickTarget;
	res.upperbound = 1 + state.tricksLeft();
	res.rwMask = rwmask;
	resl.push_back(res);
}


// Update when miss trick target
void cache_t::update_miss(const gamestate_t& state, player_t pl, uint64 rwmask, uint trickTarget) 
{
	cache_resl& resl = data[state.nCardsPlayed/4][pl][state.uSuitLengths];
	result res;
	res.cardsLeft = state.mCardsLeft;
	res.lowerbound = 0;
	res.upperbound = trickTarget;
	res.rwMask = rwmask;
	resl.push_back(res);
}


// Clear all the data
void cache_t::clear() 
{
	for (int t = 0; t < 13; t++)
		for (int p = 0; p < 4; p++)
			data[t][p].clear();
}
