// This file is part of FreeFinesse, a double-dummy analyzer (c) Ed Colley, 2006
// It is made available under the GPL; see the file COPYING for details

//
//  Cache for storing results to avoid repeat computation; this is used
//  by the analyzer
//

class cache_t {
public:
	// Check the cache for a given target. Returns -1 (miss), +1 (hit) or 0 (don't know)
	// If a hit is found, updates the rwmask parameter with the rwmask from the cache
	int check(const gamestate_t&, player_t, uint trickTarget, uint64& rwmask);

	// Update when successfully hit trick target
	void update_hit(const gamestate_t&, player_t, uint64 rwmask, uint trickTarget);

	// Update when miss trick target
	void update_miss(const gamestate_t&, player_t, uint64 rwmask, uint trickTarget);

	// Clear all the data
	void clear();
	
private:

	// Implementation details
	struct result {
		uint64 cardsLeft;		// 1 if played, else 0 (in deck order)
		uint64 rwMask;			// 1 if takes a trick, else 0 (in deck order)
		uint8 upperbound;		// min failed number of tricks
		uint8 lowerbound;		// max succeeded number of tricks
	};

	typedef std::vector<result> cache_resl;
	typedef std::map<uint64, cache_resl> data_t;

	data_t data[14][4];			// [tricks played][player on lead]
};
