// This file is part of FreeFinesse, a double-dummy analyzer (c) Ed Colley, 2006
// It is made available under the GPL; see the file COPYING for details

//
//  This file defines the interfaces between the various bits of the program
//


// Card states - this is the information the analyzer communicates to the controller
struct CardState {
	enum cs_state { cs_unplayed, cs_played, cs_notdealt, cs_numStates };
	player_t player;			// the player who holds the card
	cs_state state;				// played or not
	int sureTricks;				// how many tricks after playing this card (-1 = don't know / irrelevant)
};


// Interface for Controller -> UI
class DealView {
public:
	virtual void setState(card_t, const CardState&) = 0;
};


// Interface for UI -> Controller
class DealController_UI {
public:
	virtual bool setDeal(const deal_t& d) = 0;
	virtual bool clickCard(card_t c) = 0;
	virtual void addView(DealView*) = 0;
	virtual void removeView(DealView*) = 0;
	virtual void analyze(bool analyzeAllOptions) = 0;
};


// Interface for Analyzer -> Controller
class DealController_Analyzer {
public:
	virtual void setSureTricks(card_t, int tricks) = 0;
};


// Factory Function
DealController_UI* createDealController();

